// CurvePoint (a point in a curve)

// Copyright (C) 1999 by Makoto Nagata, MicroNova <mnagata@micronova.com>.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
// Visit http://www.micronova.com for updates

package com.micronova.math;

import netscape.util.*;

/**

   CurvePoint represents a point on a Curve.  CurvePoints are compared using their x values; i.e., two CurvePoints p and q are equal if and only if p.x == q.x, and p is 'smaller' than q if and only if p.x < q.x.

   * @author Makoto Nagata, MicroNova
   * @version 1.01
   * @see com.micronova.math.Curve 
   * @see com.micronova.math.SegmentedCurve

*/

public class CurvePoint implements Comparable, Codable
{
    /** x value of this CurvePoint. */

    public double x;

    /** y value of this CurvePoint. */

    public double y;

    /** Construcs a new CurvePoint (x, y). */

    public CurvePoint(double x, double y)
    {
        this.x = x;
        this.y = y;
    }

    /** Constructs a new CurvePoint (0, 0). */

    public CurvePoint()
    {
        this(0.0, 0.0);
    }

    /** Returns true if object is an instance of CurvePoint and this.x is equal to object.x. */

    public final boolean equals(Object object)
    {
        if (object instanceof CurvePoint)
        {
            return (this.x == ((CurvePoint)object).x);
        }
        else
        {
            return false;
        }
    }

    /** Defines the sorting ordering; for two CurvePoints p and q, 
        p is 'smaller' than q if and only if p.x < q.x */

    public final int compareTo(Object object)
    {
        CurvePoint other = (CurvePoint)object;

        if (this.x < other.x)
        {
            return -1;
        }
        else if (this.x == other.x)
        {
            return 0;
        }
        else
        {
            return 1;
        }
    }

    /** Describes the class information. */

    public void describeClassInfo(ClassInfo info)
    {
        info.addClass("com.micronova.math.CurvePoint", 1);
        info.addField("x", DOUBLE_TYPE);
        info.addField("y", DOUBLE_TYPE);
    }

    /** Encodes the instance. */

    public void encode(Encoder encoder) throws CodingException
    {
        encoder.encodeDouble("x", x);
        encoder.encodeDouble("y", y);
    }

    /** Decodes the instance. */

    public void decode(Decoder decoder) throws CodingException
    {
        x = decoder.decodeDouble("x");
        y = decoder.decodeDouble("y");
    }

    /** Finishes decoding. */

    public void finishDecoding()
    {
    }

    /** Returns a string serialization. */

    public String toString()
    {
        return "(" + x + ", " + y + ")";
    }
}









