// Numerical Equation Solvers (BiSection/Newton-Raphson/Linear)

// Copyright (C) 1999 by Makoto Nagata, MicroNova <mnagata@micronova.com>.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
// Visit http://www.micronova.com for updates


package com.micronova.math;

/**
   
   Numerical Equation Solvers (Bisection/Newton-Raphson/Linear).

   *@author Makoto Nagata, MicroNova
   *@version 1.01
   
*/

public class CurveSolver
{
    /**

       Solves an equation using Bisection method.  
       For a Curve f, this tries to find a value x such that |f(x) - target| <= accuracy in a range of x values [xLow, xHigh].  
       'NoSolution' exception is thrown if no solution is found after maxLoop tries.

       *@exception com.micronova.math.NoSolution No Solution Found

    */
    
    public final static double useBisection(Curve f, double xLow, double xHigh, double target, double accuracy, int maxLoop) throws NoSolution
    {
        return useBisection(f, xLow, f.getValueOf(xLow), xHigh, f.getValueOf(xHigh), target, accuracy, maxLoop);
    }

    /** 

        Solves an equation using Bisection method.  yLow must be f(xLow), and yHigh must be f(xHigh).  Use this version if f(xLow) and f(xHigh) are already computed.

        *@exception com.micronova.math.NoSolution No Solution Found

    */

    public final static double useBisection(Curve f, double xLow, double yLow, double xHigh, double yHigh, double target, double accuracy, int maxLoop) throws NoSolution
    {
        double x = xLow;

        for (int i = 0; i < maxLoop; i ++)
        {
            x = xLow + (target - yLow) * (xHigh - xLow) / (yHigh - yLow);

            double y = f.getValueOf(x);

            if (y > target)
            {
                if (y - target <= accuracy)
                {
                    break;
                }
                else
                {
                    xHigh = x;
                    yHigh = y;
                }
            }
            else
            {
                if (target - y <= accuracy)
                {
                    return x;
                }
                else
                {
                    xLow = x;
                    yLow = y;
                }
            }
        }

        throw new NoSolution();
    }

    /**

       Solves an equation using Newton-Raphson method.  
       Given a DeltaCurve f, this tries to find a value x such that |f(x) - target| <= accuracy, starting from an initial point xInit.  
       The differential should not be zero.  
       'NoSolution' exception is thrown if no solution is found after maxLoop tries.

       *@exception com.micronova.math.NoSolution No Solution Found

    */

    public final static double useNewtonRaphson(DeltaCurve f, double xInit, double target, double accuracy, int maxLoop) throws NoSolution
    {
        double x = xInit;

        for (int i = 0; i < maxLoop; i ++)
        {
            double y = f.getValueOf(x);

            double error = y - target;
        
            double absError = (error > 0.0) ? error : - error;

            if (absError <= accuracy)
            {
                return x;
            }
            else
            {
                x = x - error / f.getDeltaOf(x);
            }
        }

        throw new NoSolution();
    }

    /** 

        Specialized Solver for LinearSegmentedCurve.  Returns the smallest x in the given range [xLow, xHigh] such that f(x) = target (there may be rounding errors).

    */

    public final static double useLinear(LinearSegmentedCurve f, double target, double xLow, double xHigh) throws NoSolution
    {
        int size = f.size();

        if (size < 2)
        {
            // for constant curve
            
            double value = f.getValueOf(xLow);
            
            if (value == target)
            {
                return xLow;
            }
            else
            {
                throw new NoSolution();
            }
        }
        else
        {
            int beginSegment = f.getSegmentIndex(xLow);
            
            if (beginSegment == 0)
            {
                CurvePoint p = (CurvePoint)f.elementAt(0);
                    
                if (p.y == target)
                {
                    return xLow;
                }
                else
                {
                    beginSegment ++;
                }
            }
            
            int endSegment = f.getSegmentIndex(xHigh);
            
            CurvePoint p = (CurvePoint)f.elementAt(beginSegment - 1);
            
            for (int s = beginSegment; s <= endSegment; s ++)
            {
                CurvePoint q;
                
                if (s == size)
                {
                    q = p;
                }
                else
                {
                    q = (CurvePoint)f.elementAt(s);
                }
                
                if (q.y == p.y)
                {
                    if (q.y == target)
                    {
                        return p.x;
                    }
                }
                else
                {
                    double z = target - p.y;

                    if (z * (q.y - target) >= 0.0)
                    {
                        double solution = z * (q.x - p.x) / (q.y - p.y) + p.x;

                        if ((solution >= xLow) && (solution <= xHigh))
                        {
                            return solution;
                        }
                    }
                }
                
                p = q;
            }
        }

        throw new NoSolution();
    }
}












